class AS7RID_FAIL(Exception):
    pass


class INVALID_PROXY_TYPE(Exception):
    pass


class USERID_REQUIRED_FOR_BATCH(Exception):
    pass


class INVALID_ARG_IN_GET_BATCH_JOB(Exception):
    pass


class UNEXPECTED_ARG(Exception):
    pass


class URL_IS_DIFFERENT_FROM_BASE_MARKET(Exception):
    pass



import requests
import aiohttp
import inspect

from . import LolzteamExceptions


class AntipublicApi:
    def __init__(self, token: str = None, proxy_type: str = None, proxy: str = None):
        """
        :param token: Your token. You can get in there -> https://zelenka.guru/account/antipublic or in antipublic app
        :param proxy_type: Your proxy type. You can use types ( Types.Proxy.socks5 or socks4,https,http )
        :param proxy: Proxy string. Example -> ip:port or login:password@ip:port
        """
        self.base_url = "https://antipublic.one"
        if proxy_type is not None:
            proxy_type = proxy_type.upper()
            if proxy_type in ["HTTPS", "HTTP", "SOCKS4", "SOCKS5"]:
                self.__proxy_type = proxy_type
                self.__proxy = proxy
            else:
                raise LolzteamExceptions.INVALID_PROXY_TYPE(
                    "Proxy type has invalid value. It can be only https,http,socks4 or socks5"
                )
        else:
            self.__proxy = None
            self.__proxy_type = None

        self.__token = token

        self.info = self.__Info(self)
        self.account_info = self.__AccountInfo(self)

    def send_request(
        self, method: str, path: str, params: dict = None, data=None, files=None
    ):
        url = self.base_url + path
        method = method.upper()
        # AntipublicApi.__auto_delay(self)
        if params is None:
            params = {}
        if self.__token is not None:
            params["key"] = f"{self.__token}"
        proxies = {}
        if self.__proxy_type is not None:
            if self.__proxy_type == "HTTP":
                proxies = {
                    "http": f"http://{self.__proxy}",
                    "https": f"http://{self.__proxy}",
                }
            elif self.__proxy_type == "HTTPS":
                proxies = {
                    "http": f"http://{self.__proxy}",
                    "https": f"https://{self.__proxy}",
                }
            if self.__proxy_type == "SOCKS4":
                proxies = {
                    "http": f"socks4://{self.__proxy}",
                    "https": f"socks4://{self.__proxy}",
                }
            if self.__proxy_type == "SOCKS5":
                proxies = {
                    "http": f"socks5://{self.__proxy}",
                    "https": f"socks5://{self.__proxy}",
                }
            else:
                raise LolzteamExceptions.INVALID_PROXY_TYPE(
                    "Proxy type has invalid value. It can be only https,http,socks4 or socks5"
                )  # How tf you get that error?
        response = requests.request(
            method=method,
            url=url,
            params=params,
            data=data,
            files=files,
            proxies=proxies,
        )
        try:
            return response.json()
        except requests.exceptions.JSONDecodeError:
            return response.text

    async def send_async_request(
        self, method: str, path: dict, params: dict = None, data=None
    ):
        url = self.base_url + path
        method = method.upper()
        if params is None:
            params = {}
        params["key"] = f"{self.__token}"
        ptd = []
        for key, value in params.items():
            if params[key] is None:
                ptd.append(key)
        for key in ptd:
            del params[key]
        proxy_schemes = {
            "HTTP": "http",
            "HTTPS": "https",
            "SOCKS4": "socks4",
            "SOCKS5": "socks5",
        }
        request_methods = [
            "GET",
            "POST",
            "PUT",
            "DELETE",
        ]
        proxy = None
        if self.__proxy_type is not None:
            if self.__proxy_type in proxy_schemes:
                proxy_scheme = proxy_schemes[self.__proxy_type]
                proxy = f"{proxy_scheme}://{self.__proxy}"
            else:
                raise LolzteamExceptions.INVALID_PROXY_TYPE(
                    "Proxy type has invalid value. It can be only https, http, socks4 or socks5"
                )

        if method in request_methods:
            async with aiohttp.ClientSession() as session:
                async with session.request(
                    method=method, url=url, params=params, data=data, proxy=proxy
                ) as response:
                    # Иначе если делать не async with request as response и если запрос будет большим, то он не вернется. (бесконечно ждать будет)
                    # Я хуй знает почему, проблема aiohttp
                    try:
                        return await response.json()
                    except Exception:
                        return response
        else:
            raise LolzteamExceptions.AS7RID_FAIL("Invalid requests method. Contact @AS7RID")

    async def send_as_async(self, func, **kwargs):
        """
        Send request as async

        :param func: Target function
        :param kwargs: Target function parameters

        :return: json server response
        """
        im_async = True
        arguments = func.__code__.co_varnames
        loc = locals()
        for arg in arguments:
            if arg != "self":
                exec(f"{arg} = None", loc)
        if True:  # Костыль для Tweak 1
            user_id = None
        for arg, value in kwargs.items():
            if arg not in arguments:
                raise LolzteamExceptions.UNEXPECTED_ARG(
                    f"Function \"{func.__name__}\" don't have \"{arg}\" parameter"
                )
            else:
                loc[arg] = value
        func_code = str(inspect.getsource(func))
        func_code = func_code.split("):\n", 1)[1]
        lines = func_code.split("\n")
        spaces = lines[0].split('"""')[0]
        for line in lines:
            if " def " in line:
                lines.remove(line)
        return_code = (
            "\n".join(lines).replace(spaces, "").split('"""')[2].split("return ")[1]
        )
        func_code = (
            "\n".join(lines).replace(spaces, "").split('"""')[2].split("return ")[0]
        )

        exec(func_code, loc)
        path = loc.get("path")
        params = loc.get("params", {})
        data = loc.get("data", {})
        method = [
            eval(i.replace("method=", ""))
            for i in return_code.split(",")
            if "method=" in i
        ][0]
        return await AntipublicApi.send_async_request(
            self=self, method=method, path=path, params=params, data=data
        )

    class __Info:
        def __init__(self, api_self):
            self.__api = api_self

        def lines_count(self):
            """
            GET https://antipublic.one/api/v2/countLines

            Get count of rows in the AntiPublic db

            :return: json server response {'count': int}
            """

            path = "/api/v2/countLines"
            return AntipublicApi.send_request(self=self.__api, method="GET", path=path)

        def lines_count_plain(self) -> str:
            """
            GET https://antipublic.one/api/v2/countLinesPlain

            Get count of rows in the AntiPublic db (raw format)

            :return: str
            """

            path = "/api/v2/countLinesPlain"
            return AntipublicApi.send_request(self=self.__api, method="GET", path=path)

        def version(self):
            """
            GET https://antipublic.one/api/v2/version

            Get current antipublic version, change log and download url

            :return: json {'filename': str, 'version': str, 'changeLog': str, 'url': str}
            """

            path = "/api/v2/version"
            return AntipublicApi.send_request(self=self.__api, method="GET", path=path)

    class __AccountInfo:
        def __init__(self, api_self):
            self.__api = api_self

        def access(self):
            """
            GET https://antipublic.one/api/v2/checkAccess

            Checks your license

            Token required

            :return: json server response {'count': int}
            """

            path = "/api/v2/checkAccess"
            return AntipublicApi.send_request(self=self.__api, method="GET", path=path)

        def queries(self):
            """
            GET https://antipublic.one/api/v2/availableQueries

            Get your available queries

            Token required

            :return: json server response {'count': int}
            """

            path = "/api/v2/availableQueries"
            return AntipublicApi.send_request(self=self.__api, method="GET", path=path)

    def check_lines(self, lines: list[str], insert: bool = None):
        """
        GET https://antipublic.one/api/v2/checkLines

        Check your lines.

        Token required
        :param lines: Lines for check, email:password or login:password
        :param insert: Upload private rows to AntiPublic db

        :return: json server response
        """
        params = {"lines": lines, "insert": insert}
        path = "/api/v2/checkLines"
        return AntipublicApi.send_request(
            self=self, method="GET", path=path, params=params
        )

    def get_passwords(self, login: str):
        """
        GET https://antipublic.one/api/v2/emailSearch

        Get passwords for login/email

        Token required
        :param login: Email or login for search.

        :return: json server response
        """
        params = {"email": login}
        path = "/api/v2/emailSearch"
        return AntipublicApi.send_request(
            self=self, method="GET", path=path, params=params
        )

    def get_passwords_plus(self, logins: list[str], limit: int = None):
        """
        GET https://antipublic.one/api/v2/emailPasswords

        Get passwords for emails. AntiPublic Plus subscription required.

        Token required
        :param logins: Email or login for search.
        :param limit: Result limit (per email)

        :return: json server response
        """
        params = {"emails": logins, "limit": limit}
        path = "/api/v2/emailPasswords"
        return AntipublicApi.send_request(
            self=self, method="GET", path=path, params=params
        )



class Proxy:
    socks5 = "SOCKS5"
    socks4 = "SOCKS4"
    http = "HTTP"
    https = "HTTPS"


class Market:
    class Categories:
        steam = "steam"
        fortnite = "fortnite"
        vk = "vkontakte"
        genshin = "genshin-impact"
        valorant = "valorant"
        lol = "league-of-legends"
        telegram = "telegram"
        diamondrp = "diamondrp"
        supercell = "supercell"
        origin = "origin"
        wot = "world-of-tanks"  # World of Tanks
        wot_blitz = "wot-blitz"
        epicgames = "epicgames"
        eft = "escape-from-tarkov"
        twitter = "twitter"  # Not released yet
        socialclub = "socialclub"
        uplay = "uplay"
        war_thunder = "war-thunder"
        discord = "discord"
        tiktok = "tiktok"
        instagram = "instagram"
        battlenet = "battlenet"
        vpn = "vpn"
        streaming_media = "cinema"
        spotify = "spotify"
        warface = "warface"
        youtube = "youtube"
        minecraft = "minecraft"

    class Categories_ID:
        steam = 1
        fortnite = 9
        vk = 2
        genshin = 17
        valorant = 13
        lol = 29
        telegram = 24
        diamondrp = 6
        supercell = 15
        origin = 3
        wot = 14
        wot_blitz = 16
        epicgames = 12
        eft = 18
        twitter = 21
        socialclub = 7
        uplay = 5
        war_thunder = 27
        discord = 22
        tiktok = 20
        instagram = 10
        battlenet = 11
        vpn = 19
        streaming_media = 23
        spotify = 26
        warface = 4
        youtube = 25
        minecraft = 28

    class Operation_types:
        income = "income"
        cost = "cost"
        refilled_balance = "refilled_balance"
        withdrawal_balance = "withdrawal_balance"
        paid_item = "paid_item"
        sold_item = "sold_item"
        money_transfer = "money_transfer"
        receiving_money = "receiving_money"
        internal_purchase = "internal_purchase"
        claim_hold = "claim_hold"
        bid = "bid"

    class Hold_Options:
        hour = "hour"
        day = "day"
        week = "week"
        month = "month"
        year = "year"

    class Currency:
        cny = "cny"
        usd = "usd"
        rub = "rub"
        eur = "eur"
        uah = "uah"
        kzt = "kzt"
        byn = "byn"
        gbp = "gbp"

    class Item_origin:
        brute = "brute"
        stealer = "stealer"
        fishing = "fishing"
        autoreg = "autoreg"
        personal = "personal"
        resale = "resale"
        retrieve = "retrieve"
        dummy = "dummy"  # Only for steam (Пустышки)

    class Guarantee:
        half_day = -1
        day = 0
        three_days = 1

    class Item_status:
        active = "active"
        paid = "paid"
        deleted = "deleted"
        awaiting = "awaiting"

    class Order:
        price_to_up = "price_to_up"
        price_to_down = "price_to_down"
        date_to_down = "pdate_to_down"
        date_to_up = "pdate_to_up"
        date_to_down_upload = "pdate_to_down_upload"
        date_to_up_upload = "pdate_to_up_upload"
        auction_expiration = "exp_auctions"

    class App_Ids:
        CS2 = 730
        PUBG = 578080
        Steam = 753
        Dota = 570
        TF2 = 440
        Rust = 252490
        Unturned = 304930
        KF2 = 232090
        DST = 322330  # Don't Starve Together


class Forum:
    class Contests:
        class Length:
            minutes = "minutes"
            hours = "hours"
            days = "days"

        class Upgrade_prizes:
            supreme = 1
            legend = 6
            antipublic = 12
            uniq = 14
            photo_leaks = 17
            auto_participation = 19

    class Order:
        default = "natural"
        oldest = "thread_create_date"
        newest = "thread_create_date_reverse"
        newest_bumped = "thread_update_date"
        oldest_bumped = "thread_update_date_reverse"
        min_views = "thread_view_count"
        max_views = "thread_view_count_reverse"
        min_posts = "thread_post_count"
        max_posts = "thread_post_count_reverse"
        min_likes = "first_post_likes"
        max_likes = "first_post_likes_reverse"

    class PostOrder:
        default = "natural"
        default_reverse = "natural_reverse"
        max_likes = "post_likes"
        min_likes = "post_likes_reverse"

    class ReplyGroups:
        staff = 0  # КФ + кураторы
        everyone = 2  # Все
        local = 21  # Местный и выше
        resident = 22  # Постоялец и выше
        expert = 23  # Эксперт и выше
        guru = 60  # Гуру и выше
        ai = 61  # ИИ и выше



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

requirements = ["requests", "aiohttp"]

setuptools.setup(
    name="LolzteamApi",
    version="1.1.24",
    author="AS7RID",
    author_email="as7ridwork@gmail.com",
    description="A library that contains all the methods of the Lolzteam API (Market/Forum/Antipublic)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/AS7RIDENIED/Lolzteam_Python_Api",
    packages=setuptools.find_packages(),
    install_requires=requirements,
    classifiers=["Programming Language :: Python :: 3.11"],
    python_requires='>=3.9'
)



class BBCODE:
    class Color:
        @staticmethod
        def black(text: str):
            return f"[color=black]{text}[/color]"

        @staticmethod
        def white(text: str):
            return f"[color=white]{text}[/color]"

        @staticmethod
        def red(text: str):
            return f"[color=red]{text}[/color]"

        @staticmethod
        def green(text: str):
            return f"[color=green]{text}[/color]"

        @staticmethod
        def blue(text: str):
            return f"[color=blue]{text}[/color]"

        @staticmethod
        def yellow(text: str):
            return f"[color=yellow]{text}[/color]"

        @staticmethod
        def purple(text: str):
            return f"[color=purple]{text}[/color]"

        @staticmethod
        def cyan(text: str):
            return f"[color=cyan]{text}[/color]"

        @staticmethod
        def magenta(text: str):
            return f"[color=magenta]{text}[/color]"

        @staticmethod
        def custom(text: str, hex_color: str):
            return f"[color={hex_color}]{text}[/color]"

    class Alignment:
        @staticmethod
        def left(text: str):
            return f"[left]{text}[/left]"

        @staticmethod
        def center(text: str):
            return f"[center]{text}[/center]"

        @staticmethod
        def right(text: str):
            return f"[right]{text}[/right]"

    class Hide:
        @staticmethod
        def club(text: str):
            return f"[club]{text}[/club]"

        @staticmethod
        def days(text: str, days: int):
            return f"[days={days}]{text}[/days]"

        @staticmethod
        def sympathies(text: str, sympathies: int):
            return f"[likes={sympathies}]{text}[/likes]"

        @staticmethod
        def from_users(text: str, user_ids: list):
            user_ids = list(map(str, user_ids))
            return f"[exceptids={','.join(user_ids)}]{text}[/exceptids]"

        @staticmethod
        def to_users(text: str, user_ids: list):
            user_ids = list(map(str, user_ids))
            return f"[userids={','.join(user_ids)}]{text}[/userids]"

    @staticmethod
    def list(text: str, ordered: bool = False):
        if ordered:
            ordered = "=1"
        else:
            ordered = ""
        formated = [f'[*]{line}\n' for line in text.split('\n')]
        return f"[LIST{ordered}]\n{''.join(formated)}[/LIST]"

    @staticmethod
    def text_size(text: str, size: int):
        return f"[size={size}]{text}[/size]"

    @staticmethod
    def bold(text: str):
        return f"[b]{text}[/b]"

    @staticmethod
    def italic(text: str):
        return f"[i]{text}[/i]"

    @staticmethod
    def strike_through(text: str):
        return f"[s]{text}[/s]"

    @staticmethod
    def underline(text: str):
        return f"[u]{text}[/u]"

    @staticmethod
    def code(text: str):
        return f"[code]{text}[/code]"

    @staticmethod
    def source_code(text: str, code_language: str = None, inline: bool = False):
        if code_language:
            if inline:
                return f"""[srci="{code_language}"]{text}[/srci]"""
            return f"""[src="{code_language}"]{text}[/src]"""
        else:
            if inline:
                return f"[srci]{text}[/srci]"
            return f"[src]{text}[/src]"

    @staticmethod
    def image(image_url: str):
        return f"[img]{image_url}[/img]"

    @staticmethod
    def media(media_url: str, site: str = None):
        if site:
            return f"[media={site}]{media_url}[/media]"
        else:
            return f"[media]{media_url}[/media]"

    @staticmethod
    def url(url: str, text: str = None):
        if text:
            return f"[url={url}]{text}[/url]"
        else:
            return f"[url]{url}[/url]"

    @staticmethod
    def email(email: str, text: str = None):
        if text:
            return f"[email={email}]{text}[/email]"
        else:
            return f"[email]{email}[/email]"

    @staticmethod
    def user(user_id: int or str):
        return f"[user={user_id}]{user_id}[/user]"

    @staticmethod
    def quote(text: str, author: str = None):
        if author:
            return f"[quote={author}]{text}[/quote]"
        else:
            return f"[quote]{text}[/quote]"

    @staticmethod
    def spoiler(text: str, spoiler_title: str = None):
        if spoiler_title:
            return f"[spoiler={spoiler_title}]{text}[/spoiler]"
        else:
            return f"[spoiler]{text}[/spoiler]"

    @staticmethod
    def visitor():
        return f"[visitor][/visitor]"

    @staticmethod
    def plain(text: str):
        return f"[plain]{text}[/plain]"

    @staticmethod
    def api(url: str, option: str = None):
        if option:
            return f"[api={option}]{url}[/api]"
        else:
            return f"[api]{url}[/api]"
    @staticmethod
    def custom(bbcode:str,text:str="",value:str=None):
        if value:
            return f"[{bbcode}={value}]{text}[/{bbcode}]"
        else:
            return f"[{bbcode}]{text}[/{bbcode}]"
